#include "client_update.h"

/*
 * This file contains fuctions that manipulate the client update list for
 * hosts in the database.  Every host in the database has a client update
 * list so that it can notify clients waiting on it that it has finished.
 * These functions use a freelist local to this file for memory.  The
 * size of the freelist is determined by the setmem function.
 *
 */

#define TRUE 1
#define FALSE 0

static Client_update freelist = NULL;

void
client_update_setmem(num)
unsigned int num;
{
    Client_update x;
    unsigned int i;

    x = (Client_update) malloc(sizeof(struct client_update) * num);
    if (!x) {
	fprintf(stderr, "Out of memory!\n");
	exit(1);
    }

    for (i=0; i < num; i++)
	x[i].next = &x[i + 1];

    x[num - 1].next = NULL;
    freelist = x;
}


int
client_update_insert(list, client)
Client_update *list;
Client client;
{
    Client_update node;

    if (!(node = client_update_getnode()))
	return FALSE;

    node->client = client;
    node->next = *list;
    *list = node;
    return TRUE;
}


void
client_update_delete(list, client)
Client_update *list;
Client client;
{
    Client_update tmp, x;

    if (*list) {
	if ((*list)->client == client) {
	    x = (*list)->next;
	    client_update_free(*list);
	    *list = x;
	}
	else {
	    for (tmp = *list; tmp->next != NULL && tmp->next->client != client;
		 tmp = tmp->next);
	    
	    if (tmp->next && tmp->next->client == client) {
		/* delete from list */
		x = tmp->next;
		tmp->next = x->next;
		client_update_free(x);
	    }
	}
    }
}
	
Client_update
client_update_getnode()
{
    Client_update x;

    x = freelist;

    if (x)
	freelist = x->next;

    return x;
}


void
client_update_free(node)
Client_update node;
{
    node->next = freelist;
    freelist = node;
}

