#include "decode_udp.h"

extern int icmp_sock, sockaddr_in_size;
extern int *broadcast_addrs;

/*
 * decode_udp
 *
 * Description: This function unpacks a sonar request packet and processes the
 *              request.
 *
 */
void decode_udp(buf,reply_addr,s,r_ln)
char *buf;
struct sockaddr_in *reply_addr;
int s,r_ln;
{
    extern Hash host_data;
    extern Client clients;
    int i, j, num_hits;
    Client client;
    struct timeval tv;
    Hash_node host;
    
    client = client_getnode();
    if (!client) {
	/* we are a bit busy, come back later */
	send_error(buf, reply_addr, s, TOO_BUSY);
	return;
    }

    /* Unpack request packet */
    bcopy(buf,client->buf,12);
    bcopy(reply_addr, &(client->addr)  ,sockaddr_in_size);
    client->repl_ln = r_ln;

    if (*((int *)buf+1) != SONAR_REQUEST) {
	client_free(client);
	send_error(buf,reply_addr,s, BAD_REQUEST);
	return;
    }
    
    buf+=12;
    client->timeout.tv_sec = ntohl(*((unsigned int *)buf));
    buf+=4;
    client->timeout.tv_usec = ntohl(*((unsigned int *)buf));
    if (client->timeout.tv_sec > MAX_WAIT)
	client->timeout.tv_sec = MAX_WAIT;
	    
    buf+=4;
    client->cnt = ntohl(*((unsigned int *)buf));
    if (client->cnt > MAX_PACK_REQS)
	client->cnt = MAX_PACK_REQS;
    
    buf+=4;
    bcopy(buf,(char *)client->reqs,client->cnt * 4);
    if (client->timeout.tv_sec == 0 && client->timeout.tv_usec == 0) {
	do_response(s, client, 0);
	client_free(client);
	return;
    }
    
    gettimeofday(&tv, (struct timezone *) 0);
    tvadd(&(client->timeout), &tv);
    num_hits = 0;
    
    /*
     * Loop through each requested host and see what data
     * is available for it.  If we have any outstanding pings,
     * old data, old host unreachable data, etc. take appropriate
     * action.
     */
    for (i=0 ; i < client->cnt ; i++) {
	/*
	 * skip if broadcast address
	 */
	for (j=0; broadcast_addrs[j] != NULL; j++) {
	    if (broadcast_addrs[j] == client->reqs[i]) {
#ifdef _DEBUG_
		fprintf(stderr, "skipping broadcast address metric request\n");
#endif
		j = -1;
		num_hits++;
		break;
	    }
	}
	if (j == -1)
	    continue;

	
	host = hash_find(host_data,hash_index(client->reqs[i]),
			 client->reqs[i]);
	if (host) {
	    /*
	     * check to see if it has a ping sent out
	     * add the client to its waiting list
	     */
	    if (OUTSTANDING_PING(host)) {
		client_update_insert(&(host->clients_waiting), client);
		send_ping(icmp_sock, host->host); /* maybe packet was dropped*/
#ifdef _DEBUG_
		fprintf(stderr,"outstanding ping\n");
#endif
	    }

	    /*
	     * check for old host unreachable data
	     */
	    else if (host->metric == 0 && host->time_obtained <
		     tv.tv_sec - HOST_UNREACH_TIMEOUT) {
		client_update_insert(&(host->clients_waiting), client);
		send_ping(icmp_sock, host->host);
#ifdef _DEBUG_
		fprintf(stderr,"host unreach timed out\taddr = %s\n", inet_ntoa(*((struct in_addr *)&host->host)));
#endif
	    }

	    /*
	     * check for old host data
	     */
	    else if (host->time_obtained < tv.tv_sec - HOST_INFO_TIMEOUT){
		client_update_insert(&(host->clients_waiting), client);
		send_ping(icmp_sock, host->host);
#ifdef _DEBUG_
		fprintf(stderr,"old data\ttime_obtained = %d\taddr = %s\n", host->time_obtained, inet_ntoa(*((struct in_addr *)&host->host)));
#endif
	    }
	    
	    /*
	     * the current data is good enough to use
	     */
	    else {
		num_hits++;
#ifdef _DEBUG_
		if (host->metric == 0)
		    fprintf(stderr, "*HIT* host unreach\n");
		else
		    fprintf(stderr,"*HIT*\n");
#endif
	    }
	}
	else {
	    /*
	     * New host
	     */
	    host = hash_getnode(host_data);
	    SET_OUTSTANDING_PING(host);
	    host->host = client->reqs[i];
	    host->clients_waiting = client_update_create();
	    client_update_insert(&(host->clients_waiting), client);
	    hash_insert(host_data, hash_index(host->host), host);
	    send_ping(icmp_sock, host->host);
	}
    }
    
    /*
     * check to see if the request is completely filled
     */
    client->reqs_left = client->cnt - num_hits;

#ifdef _DEBUG_
    fprintf(stderr,"waiting on: %d\n", client->reqs_left);
#endif

    if (client->reqs_left)
	client_insert(clients, client);
    else {
	do_response(s, client, 0);
	client_free(client);
    }
}
