#include "recv_icmp.h"

extern int ident;

/*
 * recv_icmp
 *
 * Description: This function gets an icmp packet from the socket and
 *              determines if it is for the current process.  If it is
 *              an echo request that is returning or a host unreachable
 *              message, it updates the host information in the database.
 *
 */
void recv_icmp(sock, s)
int sock, s;
{
    int iphdrlen;
    struct ip *ip;
    struct icmp *icp;
    int now_time;
    struct timeval tv;
    static struct sockaddr_in from;
    static u_char recvpack[4096];
    u_char *buf;
    int cc, fromlen;

#ifdef _DEBUG_
    fprintf(stderr, "receiving ICMP packet\n");
#endif
    
    buf = recvpack;
    
    fromlen = sizeof(from);
    if ((cc = recvfrom(sock, recvpack, 4096, 0, (struct sockaddr *) &from,
		       &fromlen)) < 0) {
	if (errno == EWOULDBLOCK)
	    return;
	perror("recvfrom error");
	return;
    }
    
    gettimeofday(&tv, NULL);
    now_time = tv.tv_sec;
    
    ip = (struct ip *) buf;
    iphdrlen = ip->ip_hl << 2;
    if (cc < iphdrlen + ICMP_MINLEN) {
	fprintf(stderr, "packet too short (%d bytes) from %s\n", cc,
		inet_ntoa(from.sin_addr));
    }
    cc -= iphdrlen;
    icp = (struct icmp *)(buf + iphdrlen);
    
    if (icp->icmp_type == ICMP_UNREACH) {
	ip = (struct ip *)(icp->icmp_data);
	icp = (struct icmp *) (ip + 1);
	if (icp->icmp_id != ident)
	    return;
#ifdef _DEBUG_
	fprintf(stderr, "host unreach\n");
#endif
	update_node((unsigned int)ip->ip_dst.s_addr, NULL, now_time, s);
	return;
    }
    
    if (icp->icmp_type == ICMP_ECHOREPLY) {
	if (icp->icmp_id != ident )
	    return;
	
	tvsub(&tv, (struct timeval *) &icp->icmp_data[0]);
	update_node(*(unsigned int *)&(from.sin_addr),&tv,now_time, s);
    }
    return;
}
